@@ -260,6 +260,9 @@ cdplayer_destroy(GtkWidget * widget, gpointer data)
 
     g_free (cd->devpath);
  
+    if (cd->prefs_dialog)
+      gtk_widget_destroy (cd->prefs_dialog);
+
     cd->devpath = NULL;
     g_free(cd);
 }
@@ -322,11 +325,14 @@ start_gtcd_cb (BonoboUIComponent *component,
 static void
 response_cb (GtkDialog *dialog, gint id, gpointer data)
 {
+    CDPlayerData *cd = data;
+
     if(id == GTK_RESPONSE_HELP){
          phelp_cb (dialog,data);
 	 return;
     }
     gtk_widget_destroy (GTK_WIDGET (dialog));
+    cd->prefs_dialog = NULL;
 }
 
 static void
@@ -338,6 +344,8 @@ activate_cb (GtkEntry     *entry,
     newpath = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
     if(newpath && strlen(newpath) > 2 && strcmp(cd->devpath, newpath))
             {
+		if(cd_try_open(cd))
+		    cdrom_stop(cd->cdrom_device);
                 cd_close(cd);
                 cd->devpath = g_strdup(newpath);
                 if (!cd_try_open(cd)) {
@@ -406,11 +414,18 @@ preferences_cb (BonoboUIComponent *component,
     GtkWidget *entry;
     gint response;
 
+    if (cd->prefs_dialog) {
+      gtk_window_present (GTK_WINDOW (cd->prefs_dialog));
+      return;
+    }
+
     dialog = gtk_dialog_new_with_buttons(_("CD Player Preferences"),
                                          NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                          GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                                          NULL);
+    cd->prefs_dialog = dialog;
+
     gtk_window_set_screen (GTK_WINDOW (dialog),
 			   gtk_widget_get_screen (cd->panel.applet));
     gtk_dialog_set_default_response(GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
@@ -451,7 +466,7 @@ preferences_cb (BonoboUIComponent *component,
     		      G_CALLBACK (set_default_device), cd);
     
     g_signal_connect (G_OBJECT (dialog), "response",
-    		      G_CALLBACK (response_cb), NULL);
+    		      G_CALLBACK (response_cb), cd);
 
     gtk_widget_show_all(dialog);
 }
@@ -463,9 +478,10 @@ help_cb (BonoboUIComponent *component,
 {
     GError *error = NULL;
 
-    egg_screen_help_display (
+    egg_help_display_on_screen (
+		"cdplayer", NULL,
 		gtk_widget_get_screen (cd->panel.applet),
-		"cdplayer", NULL, &error);
+		&error);
 
     if (error) { /* FIXME: the user needs to see this */
         g_warning ("help error: %s\n", error->message);
@@ -804,6 +820,7 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
     CDPlayerData *cd = data;
     cdrom_device_status_t stat;
     int status;
+    int ret;
 
     if(!cd_try_open(cd))
         return;
@@ -824,16 +841,33 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
 	    cdplayer_update_play_pause_button (cd, PAUSE_IMAGE);
         case DISC_ERROR:
             cdrom_read_track_info(cd->cdrom_device);
-            cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
+            ret = cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
                    cd->cdrom_device->track1);
             break;
         }
     } else if(status == DISC_TRAY_OPEN) {
         cdrom_load(cd->cdrom_device);
         cdrom_read_track_info(cd->cdrom_device);
-        cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
+        ret = cdrom_play(cd->cdrom_device, cd->cdrom_device->track0,
                cd->cdrom_device->track1);
     }
+
+    if (ret == DISC_DEVICE_BUSY) {
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Audio Device is busy, or being used by another application"),
+						 NULL);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (cd->panel.applet));
+		g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+		                          G_CALLBACK (gtk_widget_destroy),
+					  GTK_OBJECT (dialog));
+
+		gtk_widget_show_all (dialog);
+    }
 }
 
 static void 
@@ -958,9 +992,10 @@ phelp_cb (GtkDialog *dialog, gpointer data)
 {
     GError *error = NULL;
 
-    egg_screen_help_display (
+    egg_help_display_on_screen (
+		"cdplayer", "cdplayer_applet_prefs",
 		gtk_window_get_screen (GTK_WINDOW (dialog)),
-		"cdplayer", "cdplayer_applet_prefs", &error);
+		&error);
     
     if (error) { /* FIXME: the user needs to see this */
         g_warning ("help error: %s\n", error->message);
